package com.zanxiang.sdk.service.Impl.pay;

import com.zanxiang.common.domain.ResultMap;
import com.zanxiang.common.enums.OrderStateEnum;
import com.zanxiang.common.enums.PayWayEnum;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.common.utils.SpringUtils;
import com.zanxiang.mybatis.entity.GamePayWay;
import com.zanxiang.sdk.domain.bo.PlatformOrderBO;
import com.zanxiang.sdk.domain.bo.ProductPayAttachParamBO;
import com.zanxiang.sdk.domain.bo.ProductPayParamBO;
import com.zanxiang.sdk.domain.dto.PlatformOrderDTO;
import com.zanxiang.sdk.service.GamePayWayService;
import com.zanxiang.sdk.service.OrderPayService;
import com.zanxiang.sdk.service.PlatformOrderService;
import lombok.extern.slf4j.Slf4j;
import org.jdom.JDOMException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.math.BigDecimal;

/**
 * 支付调用
 *
 * @author xufeng
 * @date 2022/6/8 16:45
 */
@Slf4j
@Service
public class PayService {
    private static final Logger logger = LoggerFactory.getLogger(PayService.class);

    @Autowired
    public PlatformOrderService platformOrderService;

    @Autowired
    public GamePayWayService gamePayWayService;

    @Value("${server.domain}")
    public String domain;


    /**
     * 自定义参数
     */
    protected ProductPayAttachParamBO attach;

    /**
     * 创建订单支付参数
     *
     * @param product
     */
    public ResultMap payCreate(ProductPayParamBO product) {
        try {
            //获取订单信息
            PlatformOrderDTO info = platformOrderService.info(product.getOutTradeNo(), product.getUserId());
            if (info == null) {
                throw new RuntimeException("订单不存在");
            }
            String gameId = String.valueOf(info.getGameId());
            //根据订单中游戏id，获取游戏的支付方式
            GamePayWay payInfo = gamePayWayService.getPayWayToOrderPay(gameId, product.getPayWay());

            product.setProductId(info.getProductId());
            product.setSubject(info.getProductName());
            product.setGameId(gameId);
            product.setTotalFee(String.valueOf(info.getAmount()));
            product.setOutTradeNo(info.getId());
            product.setFrontUrl("");
            product.setConfig(payInfo.getPayConfig());
            ProductPayAttachParamBO attachBO = new ProductPayAttachParamBO();
            attachBO.setUserId(product.getUserId());
            attachBO.setOrderId(product.getOutTradeNo());
            attachBO.setPayWay(product.getPayWay());
            attachBO.setGamePayWayId(payInfo.getId());
            product.setAttach(attachBO);
            this.attach = attachBO;
            String payCode = PayWayEnum.getCodeByNum(product.getPayWay());
            OrderPayService device = getDevice(payCode);
            ResultMap result = device.create(product);
            logger.info("订单支付参数生成 ProductPayParamBO:{},result:{}", product, result);
            return result;
        } catch (RuntimeException e) {
            e.printStackTrace();
            logger.info("订单支付参数生成异常 ProductPayParamBO:{},e:{}", product, e);
            return ResultMap.error(e.getMessage());
        }
    }

    /**
     * 支付异步回调
     *
     * @param request
     * @param response
     * @return
     */
    public String notify(HttpServletRequest request, HttpServletResponse response, Integer payType) throws IOException, JDOMException {
        String payCode = PayWayEnum.getCodeByNum(payType);
        OrderPayService device = getDevice(payCode);
        return device.notify(request, response);
    }

    /**
     * 同步消息通知
     *
     * @param request
     * @return
     */
    public ResultMap synNotify(HttpServletRequest request, Integer payType) {
        String payCode = PayWayEnum.getCodeByNum(payType);
        OrderPayService device = getDevice(payCode);
        return device.synNotify(request);
    }

    /**
     * 获取支付驱动方式
     *
     * @param payCode
     * @return
     */
    private static OrderPayService getDevice(String payCode) {
        try {
            return SpringUtils.getBean(payCode + "Service");
        } catch (BeansException e) {
            throw new BaseException("支付方式不存在");
        }
    }

    /**
     * 获取支付配置
     *
     * @param gameId    游戏id
     * @param payWay    支付方式
     * @param payDerive 支付设备
     */
    public String getPayConfig(String gameId, Integer payWay, Integer payDerive) {
        GamePayWay payInfo = gamePayWayService.getPayWayToOrderPay(gameId, payWay);
        this.attach.setGamePayWayId(payInfo.getId());
        return payInfo.getPayConfig();
    }

    /**
     * 获取支付配置
     *
     * @param gamePayWay 支付方式id
     */
    public String getPayConfig(String gamePayWay) {
        return gamePayWayService.getInfo(gamePayWay).getPayConfig();
    }

    /**
     * 支付成功逻辑
     *
     * @param orderNo         订单号
     * @param realAmount      真实金额
     * @param merchantOrderNo 订单流水号
     * @param gamePayWayId    具体游戏支付方式id
     * @return
     */
    public Boolean paySuccess(String orderNo, String realAmount, String merchantOrderNo, String gamePayWayId) {
        PlatformOrderBO bo = new PlatformOrderBO();
        bo.setId(orderNo);
        bo.setRealAmount(new BigDecimal(realAmount));
        bo.setMerchantOrderNo(merchantOrderNo);
        bo.setStatus(OrderStateEnum.SUCCESS.getCode());
        bo.setGamePaywayId(gamePayWayId);
        return platformOrderService.pay(bo);
    }
}
